; 68HC908JB8 USB HID Demonstration Application
; (C)opywrite P&E Microcomputer Systems, 2000
;
; You may use this code freely as long as this copyright notice
; and website address is included. Visit us at www.pemicro.com

; This application is meant to demonstrate a framework for an application
; running on the 68HC908JB8. It demonstrates a simple HID interface for a
; USB device and uses interrupts from the USB peripheral. The HID interface
; is supported through standard API calls in Windows 98 and Windows 2000.
; It is not supported under Windows NT without speciality drivers. For more
; information on USB, please visit the USB developers website at www.usb.org
; and download the USB specification revision 1.1 and the HID device class
; specification version 1.1 for more details.

; Application Information:

; (1) Endpoints 0 and 1 from the USB peripheral are utilized by this code.
;     All control transactions occur through endpoint 0. The lookup table
;     information dictates the capabilities of the device and is read by
;     the PC upon startup through GET_DESCRIPTOR device requests.
; (2) The state of the device is kept in the USB_State variable. The device
;     starts in the POWERED state. When a USB reset is detected, the device
;     is placed into the DEFAULT state. (Note that a USB reset should only be
;     allowed to force an interrupt for a HC908JB8 device on the ICS08JB board
;     since a RESET will place the device back into monitor mode.) When the
;     device receives a SET_ADDRESS standard device request with a valid
;     address, the device is placed into the ADDRESSED state. Finally, when a
;     SET_CONFIGURATION standard device request is sent, the device is placed
;     into the CONFIGURED state.
; (3) When the device is configured, endpoint 1 is enabled for interrupt
;     transfers from the PC. Endpoint 1 is capable of only sending information
;     out from it. Therefore, only IN packets are accepted at the endpoint.
;     The report descriptor sets up the endpoint to transfer only 1 byte of
;     data. So when a IN packet comes from the PC, the device sends out the
;     data read from the Port B input pins. Note that the internal pullups are
;     enabled for PORTB in the code (POCR), so the default input value sent
;     through endpoint 1 should be $FF.
; (4) The HC908JB8 device is assumed to have a 3.00 MHz internal frequency.
;     The COP watchdog is disabled through the CONFIG register.

; To run this application, you first need to assemble it with CASM08Z
; by pushing the F4 key on your keyboard.

; To simulate this application in the ICS08JBZ in-circuit simulator,
; simply run the ICS08JBZ and type "LOAD DEMOJB8" in the status window.
; The PC will automatically be loaded with the reset vector. Use the
; BR command to set breakpoints in the USB routines. Use the macro file
; DEMOJB8.MAC to simulate the HID process from a PC for the demo
; application by typing "MACRO DEMOJB8" in the status window. Since
; communication for the USB peripheral requires timing too fast to be
; handled through the MON08 serial interface, only ICS08JBZ commands
; (USBIN, USBOUT, USBCLR, USBRESET) can be used to simulate USB
; peripheral operation. Therefore the USB connector on the ICS08 board
; is not used by the in-circuit simulator and you do not need to attach
; a USB cable from the board to a PC while running the demo application
; in ICS08JBZ.

; To run this application realtime, you will have to use the PROG08SZ
; application to program the S19 into the JB8's on-chip flash block.
; Once you have done this, run the ICD08SZ in-circuit debugger. First,
; you should load the debug information for the application now programmed
; into the flash. Type "LOADMAP DEMOJB8.MAP" in the status window.  Only
; the first breakpoint set is valid in flash. To reset the first breakpoint,
; use the NOBR command before you set the breakpoint. If you do a GO, the
; part will be running in realtime. Note that if you break during the USB
; operation, you may need to reset the device to resynchronize to the PC
; since the device is considered timed-out after a period of time.
; If you are running under Windows 98 or Windows 2000 and attach a USB cable
; from a PC into the USB connector on the ICS08JB board, the operating system
; will detect a "new hardware device" and ask for an INF driver file. At this
; point, choose the INF file (PEDRVUSB.INF) found in the directory where the
; ICS08JB8 software is installed. When the device is detected correctly by the
; operating system, a new device type will appear under the device manager
; called "Human Interface Devices".
; Now, run the sample program (TESTZ.EXE) included in the installation. Press
; the "Find Device" button first to find the device with the given Vendor ID
; and Product ID. Then press the "Read Data" button to read data from the device
; for Port B through the USB connection. Source code for the sample program is
; included for Visual C++ (version 6.0) and Delphi (version 4.0).

; Here is the sample application...

$SET    USE_JB8_PULLUP      ;use the 908JB8 1.5K internal pullup on D-
; $SETNOT  USE_JB8_PULLUP

$pagewidth 120t

RAMStart       equ    $0040               ; start of RAM for JB8
RomStart       equ    $DC00               ; start of Flash for JB8
VectorStart    equ    $FFF0               ; start of Vectors for JB8

$Include 'jb8regs.inc'

    org RamStart
Setup_Packet     ds      8      ; array for SETUP packet
bmReqType      equ    Setup_packet        ; Characteristic of Request
bRequest       equ    {Setup_packet+1}    ; Request Code
wValueL        equ    {Setup_packet+2}    ; Low byte Value Field
wValueH        equ    {Setup_packet+3}    ; High byte Value Field
wIndexL        equ    {Setup_packet+4}    ; Low byte Index Field
wIndexH        equ    {Setup_packet+5}    ; High byte Index Field
wLengthL       equ    {Setup_packet+6}    ; Low byte Length Field
wLengthH       equ    {Setup_packet+7}    ; High byte Length Field

USB_State     ds      1         ; Device State Byte
POWERED       equ     0         ; USB_State=POWERED
DEFAULT       equ     1         ; USB_State=DEFAULT
ADDRESSED     equ     2         ; USB_State=ADDRESSED
CONFIGURED    equ     3         ; USB_State=CONFIGURED

SET_ADDRESS   equ     5         ; Standard Request code for SET_ADDRESS
GET_DESC      equ     6         ; Standard Request code for GET_DESCRIPTOR
SET_CONFIG    equ     9         ; Standard Request code for SET_CONFIGURATION

addrset       ds      1         ; bit 7=1 indicates device address has
                                ; been received, bits 6-0 = address
control       ds      1         ; type of transfer in progress
descptr       ds      2         ; pointer to descriptor being sent
descendptr    ds      2         ; end pointer to descriptor being sent
buffptr       ds      2         ; index into the USB data buffer
tptr          ds      2         ; temporary pointer for storage

    org RomStart

* The following descriptors give the information to the PC what type of
* USB device this is and what its capabilities are. They are retrieved
* during the configuration phase.

* Note that the Vendor and Product IDs specified in this demo are invalid
* USB IDs and are given for demonstration purposes only!

* Device Descriptor
Dev_Desc:
       db  {DDesc_End-Dev_Desc} ; Descriptor Length
       db  $01                  ; Descriptor Type (Device)
       db  $10,$01              ; USB specification Release (1.10)
       db  $00                  ; Class Code
       db  $00                  ; Subclass Code
       db  $00                  ; Protocol Code
       db  $08                  ; Maximum Packet Size for EP0 (8 bytes)
       db  $22,$11              ; Vendor ID=none
       db  $44,$33              ; Product ID=none
       db  $00,$01              ; Device Release Number (1.00)
       db  $01                  ; Index to Manufacturer String Descriptor
       db  $02                  ; Index to Product String Descriptor
       db  $00                  ; Index to Device Serial Number String Descriptor
       db  $01                  ; Number of possible configurations (1)
DDesc_End:

* Configuration Descriptor
Con_Desc:
       db  {CDesc_End-Con_Desc} ; Descriptor Length
       db  $02                  ; Descriptor Type (Configuration)
       db  {E1Desc_End-Con_Desc},$00 ; Total data length (Config-Interface-EP)
       db  $01                  ; Interfaces supported
       db  $01                  ; Configuration Value
       db  $00                  ; Index to String Descriptor
       db  $C0                  ; Self powered
       db  $00                  ; Maximum power consumption=0mA (not applicable)
CDesc_End:

* Interface Descriptor
Int_Desc:
       db  {IDesc_End-Int_Desc} ; Descriptor Length
       db  $04                  ; Descriptor Type (Interface)
       db  $00                  ; Number of Interface
       db  $00                  ; No alternate setting
       db  $01                  ; Number of endpoints
       db  $03                  ; Class Code (HID)
       db  $00                  ; Subclass Code
       db  $00                  ; Protocol Code
       db  $00                  ; Index to String Descriptor
IDesc_End:

* HID Descriptor
HID_Desc:
       db  {HDesc_End-HID_Desc} ; Descriptor Length
       db  $21                  ; Descriptor Type (HID)
       db  $00,$01              ; HID Class Release (1.00)
       db  $00                  ; Country Code=$00
       db  $01                  ; number of HID class descriptors
       db  $22                  ; Class Descriptor Type (REPORT)
       db  {RDesc_End-Rep_Desc},$00 ; length of report descriptor
HDesc_End:

* Endpoint Descriptor
Endp1_Desc:
       db  {E1Desc_End-Endp1_Desc} ; Descriptor Length
       db  $05                  ; Descriptor Type (Endpoint)
       db  $81                  ; Endpoint Address and Direction (#1,IN)
       db  $03                  ; Endpoint Attribute (Interrupt)
       db  $01,$00              ; Maximum Packet Size for EP1 (1 byte)
       db  $FF                  ; Polling Interval=255[ms]
E1Desc_End:

* Report Descriptor
Rep_Desc:
       db  $06,$00,$FF          ; Usage Page (vendor defined)
       db  $09,$01              ; Usage (vendor defined)

       db  $A1,$01              ; Collection (Application)
       db  $09,$02              ; Usage (vendor defined)

; Input report
       db  $09,$03              ; Usage (vendor defined)
       db  $15,$00              ; Logical Minimum ($00)
       db  $26,$FF,$00          ; Logical Maximum ($FF)
       db  $75,$08              ; Report Size (8 bits)
       db  $95,$01              ; Report Count (1 field)
       db  $81,$02              ; Input (Data, Variable, Absolute)

       db  $C0                  ; End Collection
RDesc_End:

***************************************************************
* Init_USB - Disables receive and transmit for all endpoints. *
*            The USB state is set to powered, where the part  *
*            is waiting for an USB reset and for it to be     *
*            addressed and configured.                        *
***************************************************************
Init_USB:
       mov   #POWERED,USB_State ; device in POWERED state
       clr   addrset            ; not addressed yet
       clr   control            ; waiting for control packets
       mov   #80,UADDR          ; enable USB, address=default
       clr   UCR0               ; disable all for Epnt 0
       clr   UCR1               ; disable all for Epnt 1
       clr   UCR2               ; disable all for Epnt 2
       mov   #3,UIR0            ; enable TXD0IE, RXD0IE
       mov   #$FF,UIR2          ; Reset all USB flags
$IF USE_JB8_PULLUP
       mov   #5,UCR3            ; enable EP1, no STALL, internal pullup
$ELSEIF
       mov   #1,UCR3            ; enable EP1, no STALL, no internal pullup
$ENDIF
       rts


**************************************************************
* MAIN_INIT - This is the point where code starts executing  *
*             after a RESET.                                 *
**************************************************************
MAIN_INIT:
       rsp
       clra
       clrx
       mov   #$21,CONFIG        ; URSTD=1 (USB reset=interrupt),
                                ; diasble COP watchdog
       mov   #$02,POCR          ; enable pullups for port B
       jsr   Init_USB           ; Initialize USB peripheral
       cli                      ; Allow interrupts to happen

main_loop:
       nop                            ; do nothing in main loop
       bra   main_loop

****************************************************************
* FORCE_STALL - A packet is received in the control stage that *
*               is not supported. So the device stalls until   *
*               new SETUP packet arrives at endpoint 0.        *
****************************************************************
FORCE_STALL:
       lda   UCR3               ; request not handled
       ora   #30                ; set OSTALL0 and ISTALL0
       sta   UCR3               ; new SETUP packet clears STALL
       rts                      ;  bits automatically

****************************************************************
* SETADDR_PROC - This procedure handles the standard request   *
*                to set the device's address.                  *
****************************************************************
SETADDR_PROC:
       ldhx  #5
SETADDR_PROC2:
       lda   {wValueH-1},x      ; if any field but ValueL<>0 then
       bne   SETADDR_STALL      ; go stall
       dbnzx SETADDR_PROC2
       lda   wValueL            ; get address value
       bmi   SETADDR_STALL      ; if greater than 7 bits, stall

       ora   #$80               ; set highest bit for flag
       sta   addrset            ; store new address in addrset
                                ; change UADDR after last IN packet comes
       mov   #SET_ADDRESS,control ; set flag for control transfer type
       mov   #3,UIR2            ; clear any TXD0F, RXD0F flag
       mov   #$B0,UCR0          ; allow IN packets, when IN packet comes
       bra   SETADDR_EXIT       ;  send DATA1 packet with 0 bytes

SETADDR_STALL:
       jsr   FORCE_STALL

SETADDR_EXIT:
       rts

****************************************************************
* GETDESC_PROC - This procedure handles the standard request   *
*                to get the device's descriptors.              *
****************************************************************
GETDESC_PROC:
       ldhx  #0                 ; clear H:X pointer
       lda   wValueH            ; check which descriptor is wanted
       cbeqa #$1,GETDEVDESC     ; is it device descriptor?
       cbeqa #$2,GETCONDESC     ; is it configuration descriptor?
       cbeqa #$21,GETHIDDESC    ; is it for HID descriptor?
       cbeqa #$22,GETREPDESC    ; is it for Report descriptor?
       jmp   GETDESC_STALL      ; else go stall

GETDEVDESC:
       lda   Dev_Desc,x         ; take device descriptor information
       sta   UE0D0,x            ; store in USB endpoint 0 data buffer
       incx
       cpx   #8                 ; all descriptors more than 8 bytes
       bne   GETDEVDESC
       ldhx  #DDesc_End         ; store end location of the descriptor
       sthx  descendptr
       ldhx  #Dev_Desc          ; store pointer to next byte in descriptor
       bra   GETDESC_END

GETCONDESC:
       lda   Con_Desc,x         ; take configuration descriptor information
       sta   UE0D0,x            ; store in USB endpoint 0 data buffer
       incx
       cpx   #8                 ; all descriptors more than 8 bytes
       bne   GETCONDESC
       ldhx  #E1Desc_End        ; store end location of the descriptor
       sthx  descendptr
       ldhx  #Con_Desc          ; store pointer to next byte in descriptor
       bra   GETDESC_END

GETHIDDESC:
       lda   HID_Desc,x         ; take HID descriptor information
       sta   UE0D0,x            ; store in USB endpoint 0 data buffer
       incx
       cpx   #8                 ; all descriptors more than 8 bytes
       bne   GETHIDDESC
       ldhx  #HDesc_End         ; store end location of the descriptor
       sthx  descendptr
       ldhx  #HID_Desc          ; store pointer to next byte in descriptor
       bra   GETDESC_END

GETREPDESC:
       lda   REP_Desc,x         ; take report descriptor information
       sta   UE0D0,x            ; store in USB endpoint 0 data buffer
       incx
       cpx   #8                 ; all descriptors more than 8 bytes
       bne   GETREPDESC
       ldhx  #RDesc_End         ; store end location of the descriptor
       sthx  descendptr
       ldhx  #REP_Desc          ; store pointer to next byte in descriptor

GETDESC_END:
       sthx  descptr
       lda   descptr+1          ; add length specified in setup packet
       add   wLengthL           ;  and store in tptr
       sta   tptr+1
       lda   descptr
       adc   wLengthH
       sta   tptr
       ldhx  tptr               ; is calculated pointer >= end of descriptor?
       cphx  descendptr         ; if so then end_pointer = end of descriptor
       bge   GETDESC_END2       ; else, end_pointer= calculated pointer
       sthx  descendptr

GETDESC_END2:
       ldhx  descptr            ; get pointer to start of descriptor
       aix   #8                 ; eight bytes sent already
       sthx  descptr            ; store current pointer
       mov   #$B8,UCR0          ; change to DATA1 sequence, SIZE=8 bytes
                                ; turn on transmitter (TX0E)
       mov   #GET_DESC,control  ; set flag for control transfer type
       bra   GETDESC_EXIT

GETDESC_STALL:
       jsr   FORCE_STALL

GETDESC_EXIT:
       rts

****************************************************************
* SETCON_PROC - This procedure handles the standard request to *
*               set the configuration of the device.           *
****************************************************************
SETCON_PROC:
       ldhx  #5
SETCON_PROC2:
       lda   {wValueH-1},x      ; if any field but ValueL<>0 then
       bne   SETCON_STALL       ; go stall
       dbnzx SETCON_PROC2
       lda   wValueL            ; get configuration value
       cmp   #01
       bne   SETCON_STALL       ; if value<>1 then stall
       mov   #SET_CONFIG,control ; set flag for control transfer type
       mov   #3,UIR2            ; clear any TXD0F, RXD0F flag
       mov   #$B0,UCR0          ; allow IN packets, when IN packet comes
       bra   SETCON_EXIT        ;  send DATA1 packet with 0 bytes

SETCON_STALL:
       jsr   FORCE_STALL

SETCON_EXIT:
       rts

****************************************************************
* SETUP_PROC - This procedure handles the SETUP packets that   *
*              come into the USB peripheral. The only standard *
*              device requests handled are SET_ADDRESS,        *
*              GET_DESCRIPTOR, and SET_CONFIGURATION.          *
****************************************************************
SETUP_PROC:
       bclr  4,UCR0             ; turn off reception from endpoint 0
       bset  0,UIR2             ; reset the RXD0F flag
       clr   control            ; clear flag for control transfer
       lda   USR0               ; check size and sequence of SETUP packet
       cmp   #48                ; is R0SEQ=0, SIZE=8?
       bne   SETUP_EXIT         ; if not then exit
       ldhx  #8
SAVE_SETUP:
       lda   {UE0D0-1},x        ; save data to array
       sta   {Setup_packet-1},x ; Setup_packet holds info
       dbnzx SAVE_SETUP
       lda   bmReqType          ; if request type is standard
       and   #$60               ; then go handle standard request
       bne   SETUP_STALL        ; otherwise, force stall

STANDARD:
       lda   bRequest           ; get request type
       cbeqa #SET_ADDRESS,SET_ADDR  ; if setting address then
                                    ; store new address and change after
                                    ; final IN packet
       cbeqa #GET_DESC,GET_DESCR    ; if getting descriptors then
                                    ; ready the next IN packets for
                                    ; the descriptor information
       cbeqa #SET_CONFIG,SET_CON    ; if setting configuration then
                                    ; move from ADDRESSED to CONFIGURED state
       bra   SETUP_STALL        ; otherwise, force stall

SET_ADDR:
       jsr   SETADDR_PROC       ; packet received was SET_ADDRESS
       bra   SETUP_EXIT         ; go exit

GET_DESCR:
       jsr   GETDESC_PROC       ; packet received was GET_DESCRIPTOR
       bra   SETUP_EXIT         ; go exit

SET_CON:
       jsr   SETCON_PROC        ; packet received was SET_CONFIGURATION
       bra   SETUP_EXIT         ; go exit

SETUP_STALL:
       jsr   FORCE_STALL        ; error, force stall on endpoint 0

SETUP_EXIT:
       bset  4,UCR0             ; turn on reception from endpoint 0
       rts

****************************************************************
* IN_PROC - This procedure handles the IN packets that come    *
*           into the USB peripheral through endpoint 0.        *
****************************************************************
IN_PROC:
       bclr  5,UCR0             ; turn off transmits from endpoint 0
       bset  1,UIR2             ; reset the TXD0F flag
       lda   control            ; is this IN packet for status stage
       cmp   #SET_ADDRESS       ;  of SET_ADDRESS?
       bne   IN_PROC2
       lda   addrset            ; then set address of device
       sta   UADDR
       ldx   #DEFAULT           ; device in DEFAULT state
       cbeqa #$80,IN_DEF        ; if address is 0 then DEFAULT state
       ldx   #ADDRESSED         ; otherwise device in ADDRESSED state
IN_DEF:
       stx   USB_State          ; set state of device
       clr   addrset            ; clear flag and address
       clr   control            ; end of control transfer
       bra   IN_EXIT

IN_PROC2:
       cmp   #GET_DESC          ; is this IN packet for data stage
       bne   IN_PROC3           ;  of GET_DESCRIPTOR?
       ldhx  #0                 ; clear index into data buffer
       sthx  buffptr
IN_SEND_DATA:
       ldhx  descptr            ; get pointer to descriptor
       lda   0,x                ; get descriptor byte
       aix   #1
       sthx  descptr            ; save pointer to descriptor
       ldhx  buffptr            ; get index to data buffer
       sta   UE0D0,x            ; place byte in buffer
       aix   #1
       sthx  buffptr            ; save index to data buffer
       ldhx  descptr            ; is it last descriptor byte?
       cphx  descendptr
       beq   LAST_DESC
       ldhx  buffptr            ; is data buffer filled?
       cphx  #8
       beq   DATAFILLED         ;  go send data
       bra   IN_SEND_DATA       ; else continue

LAST_DESC:
       clr   control            ; if so then end of data transfer
DATAFILLED:
       lda   UCR0               ; take USB control register 0
       and   #$F0               ; mask off top 4 bits
       ora   #$20               ; enable transmits from endpoint 0
       eor   #$80               ; toggle DATA sequence bit
       ora   buffptr+1          ; set the size of the buffer
       sta   UCR0
       bra   IN_EXIT

IN_PROC3:
       cmp   #SET_CONFIG        ; is this IN packet for status stage
       bne   IN_EXIT            ;  of SET_CONFIGURATION?
       mov   #CONFIGURED,USB_State ; device in CONFIGURED state
       clr   control            ; end of control transfer
       lda   ptb                ; get Port B value
       sta   UE1D0              ; place in endpoint 1 data buffer
       mov   #21,UCR1           ; turn on endpoint 1 transmit, size=1 byte
                                ; Sequence=0
       bset  3,UIR0             ; enable interrupt for endpoint 1 transmit

IN_EXIT:
       rts

****************************************************************
* OUT_PROC - This procedure handles the OUT packets that are   *
*            sent by the USB peripheral through endpoint 0.    *
****************************************************************
OUT_PROC:
       bclr  4,UCR0             ; turn off reception from endpoint 0
       bset  0,UIR2             ; reset the RXD0F flag
                                ; all OUT packets received for endpoint 0
                                ; should be for status stage of requests
OUT_EXIT:
       bset  4,UCR0             ; turn on reception from endpoint 0
       rts

**************************************************************
* USB_ISR - USB Interrupt Service Routine.                   *
*           Interrupts here if USB reset, packet is received *
*           in endpoint 0, or packet is transmitted from     *
*           endpoint 0 or 1.                                 *
**************************************************************
USB_ISR:
       brclr 6,UIR1,UISR_RXD    ; Is it USB reset?
       mov   #DEFAULT,USB_State ; device in DEFAULT state
       mov   #10,UCR0           ; turn on RX0
       mov   #80,UADDR          ; enable USB, address=default
       clr   UCR1               ; disable all for Epnt 1
       mov   #3,UIR0            ; enable TXD0IE, RXD0IE
       mov   #40,UIR2           ; reset USB reset flag
       bra   UISR_EXIT          ; exit interrupt

UISR_RXD:
       brclr 0,UIR1,UISR_TXD    ; Is it SETUP/OUT packet received?
       brclr 6,USR0,UISR_RXD2   ; is it SETUP packet?
       jsr   SETUP_PROC         ; handle SETUP packet
       bra   UISR_EXIT          ; exit interrupt

UISR_RXD2:
       jsr   OUT_PROC           ; handle OUT packet
       bra   UISR_EXIT          ; exit interrupt

UISR_TXD:
       brclr 1,UIR1,UISR_TXD1   ; Is packet sent from IN packet for endpoint 0?
       jsr   IN_PROC            ; handle IN packet for endpoint 0
       bra   UISR_EXIT          ; exit interrupt

UISR_TXD1:
       brclr 3,UIR1,UISR_EXIT   ; Is packet sent from IN packet for endpoint 1?
       lda   ptb                ; get Port B value
       sta   UE1D0              ; place in endpoint 1 data buffer
       lda   UCR1               ; toggle TX1 data sequence
       eor   #80
       sta   UCR1
       bset  3,UIR2             ; reset the TXD1F flag

UISR_EXIT:
       rti

**************************************************************
* DUMMY_ISR - Dummy Interrupt Service Routine.               *
*             Just does a return from interrupt.             *
**************************************************************
DUMMY_ISR:
       rti                       ; simple return

**************************************************************
* Vectors - Specifying Reset and USB Interrupt Routines    *
**************************************************************
       org   VectorStart

       dw    dummy_isr          ; Keyboard Vector
       dw    dummy_isr          ; TIM Overflow Vector
       dw    dummy_isr          ; TIM Channel 1 Vector
       dw    dummy_isr          ; TIM Channel 0 Vector
       dw    dummy_isr          ; ~IRQ1
       dw    usb_isr            ; USB Vector
       dw    dummy_isr          ; SWI Vector
       dw    main_init          ; Reset Vector


; 68HC908JB8 USB HID Demonstration Application
; (C)opywrite P&E Microcomputer Systems, 2000
;
; You may use this code freely as long as this copyright notice
; and website address is included. Visit us at www.pemicro.com
